""" Copyright start
  Copyright (C) 2008 - 2022 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import logging
import os
import json
import inspect

from django.conf import settings

logger = logging.getLogger(__name__)

try:
    from expression_builder.custom.custom_functions import func_map
except Exception as e:
    func_map = {}
    logger.warn('Error while importing custom expression: %s', str(e))


def prepare_exp_list(*args, **kwargs):
    expressions = {}
    dir_expression = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'expressions')
    dir_custom = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'custom')
    files_to_iterate = [os.path.join(dir_expression, file) for file in os.listdir(dir_expression) if
                        file.endswith('.json')]
    files_to_iterate.extend(
        [os.path.join(dir_custom, file) for file in os.listdir(dir_custom) if file.endswith('.json')])
    for filename in files_to_iterate:
        try:
            with open(filename) as json_data:
                output = json.load(json_data)

                if output.get('name') in expressions:
                    expressions['name'].get('operations', []).update(output.get('operations'))
                else:
                    expressions[output.get('name')] = output

                # if filename.endswith('custom_expressions.json'):
                #     # Populate custom function in other expressions
                #     operations = populate_custom_func()
                #     output['operations'].extend(operations)
        except Exception as e:
            logger.error('Error while reading file: %s, Error: %s', filename, str(e))

    return expressions


def populate_custom_func():
    operations = []
    operation_template = {
        'name': 'func_name',
        'jexp': '{0}()',
        'description': '',
        'params': {
        }
    }

    for k, v in func_map.items():
        operation_template['name'] = k
        jexp = '{0}('.format(k)
        operation_template['title'] = k
        res = inspect.getfullargspec(v)
        for arg in res.args:
            jexp += '{{params.%s}},' % arg
            operation_template['params'][arg] = {
                'name': arg,
                'description': arg,
                'title': arg,
                'editable': True,
                'type': 'text',
                'value': '',
                'required': False
            }
        jexp += ')'
        operation_template['jexp'] = jexp
        operations.append(operation_template)
    return operations
