""" Copyright start
  Copyright (C) 2008 - 2022 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import sys
import logging
from retry import retry
from threading import Thread
from django.apps import AppConfig
from postman.utils.rmq_utility import prepare_intra_cyops_config, prepare_intra_cyops_workflow_execute_config, \
    get_redundant_queues, _prepare_mq_config_dict
from django.conf import settings
from workflow.rabbitmq_utils import RabbitMQPublisher

logger = logging.getLogger(__name__)


class PostmanConfig(AppConfig):
    name = 'postman'

    def check_exit_command(self):
        commands = ['makemigrations', 'migrate', 'createcachetable', 'collectstatic', 'global_variables',
                    'migrate_config', 'default_schedules']
        for command in commands:
            if command in sys.argv:
                return True

    def ready(self):
        if self.check_exit_command():
            return
        load_agents()
        if '/opt/cyops-workflow/.env/bin/gunicorn' in sys.argv:
            # Need to delete queues with diff self id
            # This helps when agent id of self node is changed
            delete_redundant_queues()
            start_workflow_consumer()
            start_workflow_execute_consumer()
            create_rmq_publisher_placeholder()


def create_rmq_publisher_placeholder():
    settings.RMQ_PUBLISHER = RabbitMQPublisher()


def start_workflow_consumer():
    config = prepare_intra_cyops_config()
    t = Thread(target=start_consumer, name='workflow consumer', args=(config,))
    t.start()
    logger.info('Started workflow consumers successfully')


def start_workflow_execute_consumer():
    config = prepare_intra_cyops_workflow_execute_config()
    t = Thread(target=start_consumer, name='workflow execute consumer', args=(config,))
    t.start()
    logger.info('Started workflow execute consumers successfully')


# indefinite retry with 10 sec interval until rmq connectivity is stored
@retry(delay=10, logger=logger)
def start_consumer(config, *args, **kwargs):
    from postman.core.broker_factory import BrokerFactory
    # keeping consumer's prefetch count to 4 so that consumers buffer is not filled with many messages
    config['prefetch_count'] = 4
    broker = BrokerFactory.get_broker_instance(config=config)
    broker.start_consumer()


def delete_redundant_queues():
    try:
        from postman.core.broker_factory import BrokerFactory
        all_redundant_queues = get_redundant_queues()
        config = _prepare_mq_config_dict()
        config['redundant_queues'] = all_redundant_queues
        broker = BrokerFactory.get_broker_instance(config=config)
        broker.delete_redundant_queues()
    except Exception as e:
        logger.warn('Error while deleting redundant queues %s', str(e))


def load_agents():
    from postman.utils.utility import init_self_agent, load_tenants
    try:
        init_self_agent()
        load_tenants()
    except Exception as e:
        logger.error('Error while loading agent details %s', str(e))

