""" Copyright start
  Copyright (C) 2008 - 2022 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
from unittest import TestCase
import json

import requests
import requests_mock

from sealab.utils import maybe_json_or_raise


@requests_mock.Mocker(kw='mock')
class MaybeJsonTestCase(TestCase):

    def setUp(self):
        self.url = 'https://www.wut.com'

    def test_200_json_response(self, **kwargs):
        mock = kwargs.get('mock')
        json_data = {'um': 'what'}
        mock.get(self.url, json=json_data, status_code=200)

        response = requests.get(self.url)
        result = maybe_json_or_raise(response)
        self.assertEqual(json_data, result)

    def test_200_text_response(self, **kwargs):
        mock = kwargs.get('mock')

        text = 'wut'
        mock.get(self.url, text=text, status_code=200)
        response = requests.get(self.url)
        result = maybe_json_or_raise(response)
        self.assertEqual(text, result)

    def test_non_2XX_response_empty(self, **kwargs):
        mock = kwargs.get('mock')
        mock.get(self.url, status_code=401)

        response = requests.get(self.url)
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            requests_msg = str(e)
        expected_msg = "{} :: {}".format(requests_msg, '')

        try:
            maybe_json_or_raise(response)
        except requests.exceptions.HTTPError as e:
            self.assertEqual(str(e), expected_msg)

    def test_non_2XX_response_json_message(self, **kwargs):
        json_data = {'err': 'something happened'}

        mock = kwargs.get('mock')
        mock.get(self.url, json=json_data, status_code=401)

        response = requests.get(self.url)
        self.assertRaises(requests.exceptions.HTTPError,
                          lambda: maybe_json_or_raise(response))

        response = requests.get(self.url)
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            requests_msg = str(e)
        expected_msg = "{} :: {}".format(requests_msg, json.dumps(json_data))

        try:
            maybe_json_or_raise(response)
        except requests.exceptions.HTTPError as e:
            self.assertEqual(str(e), expected_msg.replace('"', "'"))

    def test_non_2XX_response_with_text(self, **kwargs):
        text = 'error str'

        mock = kwargs.get('mock')
        mock.get(self.url, text=text, status_code=403)

        response = requests.get(self.url)
        self.assertRaises(requests.exceptions.HTTPError,
                          lambda: maybe_json_or_raise(response))

        response = requests.get(self.url)
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            requests_msg = str(e)
        expected_msg = "{} :: {}".format(requests_msg, text)

        try:
            maybe_json_or_raise(response)
        except requests.exceptions.HTTPError as e:
            self.assertEqual(str(e), expected_msg)
